/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX’ source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "mainwindow.h"

#include "deviation.h"
#include "ui_deviation.h"

Deviation::Deviation(MainWindow *mw, DataCalculations *datacal, DataStorage *dst, SettingsStorage *sst, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::Deviation){

    ui->setupUi(this);

    //own code

    this->mw = mw;
    this->datacal = datacal;
    this->dst = dst;
    this->sst = sst;


    sst->minSize_Deviation = new QPoint(ui->mainWidget->width(), ui->mainWidget->height());

    setSize();

    grid = new QwtPlotGrid();
    grid->enableX(true);
    grid->enableXMin(true);
    grid->enableY(true);
    grid->enableYMin(true);
    grid->attach(ui->plot);

    curveErr = new QwtPlotCurve("deviations");
    curveErr->setStyle(QwtPlotCurve::Dots);
    colorErr = new QColor(Qt::darkGreen);
    brushErr = new QBrush (*colorErr);
    penErr = new QPen(*colorErr);
    sizeErr = new QSize(sst->prgset->thicknessOfCurveDots,sst->prgset->thicknessOfCurveDots);
    symbolErr = new QwtSymbol(QwtSymbol::Ellipse, *brushErr, *penErr, *sizeErr);
    curveErr->setSymbol(symbolErr);

    curveInterpol = new QwtPlotCurve("fit");
    curveInterpol->setStyle(QwtPlotCurve::Lines);
    colorInterpol = new QColor(255, 165, 0);
    penInterpol = new QPen(*colorInterpol);
    penInterpol->setWidth(sst->prgset->thicknessOfCurveLines);
    curveInterpol->setPen(*penInterpol);
    curveInterpol->setCurveFitter(new QwtSplineCurveFitter());

    ui->plot->setAxisTitle(QwtPlot::yLeft,tr("relative Abweichung in Prozent"));
    ui->plot->setAxisTitle(QwtPlot::xBottom,"s [1/Å]");
    ui->plot->setAxisScale(QwtPlot::xBottom, 0, 1);
    ui->plot->setAxisAutoScale(QwtPlot::yLeft,true);

    updateCurves();

    canClose = false;

}

Deviation::~Deviation(){
    delete ui;

    curveInterpol->detach();
    delete curveInterpol;
    delete penInterpol;
    delete colorInterpol;

    curveErr->detach();
    delete curveErr;
    delete brushErr;
    delete penErr;
    delete sizeErr;
    delete colorErr;

    grid->detach();
    delete grid;
}


void Deviation::setCloseAllowed(){
    canClose = true;
}

void Deviation::setFollowChanges(bool value){
    sst->mustSave = true;
    ui->checkBox_followChanges->setChecked(value);
    dst->prgpar->followChanges = value;
    if (value)
        ui->button_update->setDisabled(true);
    else
        ui->button_update->setEnabled(true);
}

void Deviation::setEnabledDisabledForDisplayOfAutoFitResults(bool displayAutoFitResults){
    ui->checkBox_followChanges->setDisabled(displayAutoFitResults);
    ui->button_update->setDisabled(displayAutoFitResults || ui->checkBox_followChanges->isChecked());
}

void Deviation::setInterpolation(bool value){
    ui->checkBox_Interpolation->setChecked(value);
    on_checkBox_Interpolation_clicked(value);
}

void Deviation::updatePlot(){
    datacal->calcDeviationData(dst->deviationData, dst->data_cor, dst->plotPars_Deviation, dst->prgpar->currentWavelength->value, dst->prgpar->useA, dst->prgpar->par_d, dst->prgpar->useGradient, dst->prgpar->useCorrAutoColl, dst->prgpar->par_r, dst->prgpar->par_delta, dst->prgpar->par_l, dst->prgpar->useQ, dst->prgpar->par_b);
    curveErr->setData(new QwtPointSeriesData(*(dst->deviationData)));
    curveInterpol->setData(new QwtPointSeriesData(*(dst->deviationData)));
    if (sst->prgset->adjustAxisIntervals)
        ui->plot->setAxisScale(QwtPlot::xBottom, dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
    ui->plot->replot();
}

void Deviation::resetPlot(){
    dst->deviationData->clear();
    curveErr->setData(new QwtPointSeriesData(*(dst->deviationData)));
    curveInterpol->setData(new QwtPointSeriesData(*(dst->deviationData)));
    if (sst->prgset->adjustAxisIntervals)
        ui->plot->setAxisScale(QwtPlot::xBottom, dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
    ui->plot->replot();
}

void Deviation::updateCurves(){
    delete sizeErr;
    sizeErr = new QSize(sst->prgset->thicknessOfCurveDots,sst->prgset->thicknessOfCurveDots);
    symbolErr = new QwtSymbol(QwtSymbol::Ellipse, *brushErr, *penErr, *sizeErr);
    curveErr->setSymbol(symbolErr);

    delete penInterpol;
    penInterpol = new QPen(*colorInterpol);
    penInterpol->setWidth(sst->prgset->thicknessOfCurveLines);
    curveInterpol->setPen(*penInterpol);

    curveErr->detach();
    curveInterpol->detach();
    switch(sst->prgset->elementInFront){
        case Enumerations::dotsInFront :
            if (ui->checkBox_Interpolation->isChecked())
                curveInterpol->attach(ui->plot);
            curveErr->attach(ui->plot);
            break;
        case Enumerations::lineInFront :
            curveErr->attach(ui->plot);
            if (ui->checkBox_Interpolation->isChecked())
                curveInterpol->attach(ui->plot);
            break;
        default:
            if (ui->checkBox_Interpolation->isChecked())
                curveInterpol->attach(ui->plot);
            curveErr->attach(ui->plot);
            break;
    }
    ui->plot->replot();
}

void Deviation::updateBorders(){
    ui->plot->setAxisScale(QwtPlot::xBottom, dst->prgpar->plotMinX, dst->prgpar->plotMaxX);
    ui->plot->replot();
}

void Deviation::on_scrollBarH_valueChanged(int value){
    ui->plot->move(-value + 25, ui->plot->y());
    ui->checkBox_followChanges->move(-value + 25, ui->checkBox_followChanges->y());
    ui->checkBox_Interpolation->move(-value + 25, ui->checkBox_followChanges->y());
    ui->button_update->move(-value + 335, ui->button_update->y());
    ui->button_SavePlotImage->move(-value + 25, ui->button_SavePlotImage->y());
    ui->button_SavePlotAsXY->move(-value + 335, ui->button_SavePlotAsXY->y());
    ui->button_Close->move(-value + 25, ui->button_Close->y());
}

void Deviation::on_scrollBarV_valueChanged(int value){
    ui->plot->move(ui->plot->x(), -value + 25);
    ui->checkBox_followChanges->move(ui->checkBox_followChanges->x(), -value + 640);
    ui->checkBox_Interpolation->move(ui->checkBox_followChanges->x(), -value + 670);
    ui->button_update->move(ui->button_update->x(), -value + 650);
    ui->button_SavePlotImage->move(ui->button_SavePlotImage->x(), -value + 720);
    ui->button_SavePlotAsXY->move(ui->button_SavePlotAsXY->x(), -value + 720);
    ui->button_Close->move(ui->button_Close->x(), -value + 770);
}

void Deviation::on_checkBox_Interpolation_clicked(bool checked){
    sst->mustSave = true;
    dst->prgpar->showInterpolation = checked;
    if (checked){
        switch(sst->prgset->elementInFront){
            case Enumerations::dotsInFront :
                curveErr->detach();
                curveInterpol->attach(ui->plot);
                curveErr->attach(ui->plot);
                break;
            case Enumerations::lineInFront :
                curveInterpol->attach(ui->plot);
                break;
            default:
                curveErr->detach();
                curveInterpol->attach(ui->plot);
                curveErr->attach(ui->plot);
                break;
        }

    }
    else
        curveInterpol->detach();
    ui->plot->replot();
}

void Deviation::on_checkBox_followChanges_clicked(bool checked){
    sst->mustSave = true;
    dst->prgpar->followChanges = checked;
    if (checked){
        ui->button_update->setDisabled(true);
        if (!(isHidden()))
            updatePlot();
    }
    else
        ui->button_update->setEnabled(true);
}

void Deviation::on_button_update_clicked(){
    updatePlot();
}

//screenshot of plot (plotWidget)
void Deviation::on_button_SavePlotImage_clicked(){
    QPixmap *plotScreenshot = new QPixmap(ui->plot->size());
    ui->plot->render(plotScreenshot);
    if ( !(dst->saveImageFile(plotScreenshot, new QString(tr("aktuellen Plot als Bild speichern")), sst->prgset->defaultDir, this)) )
         AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern des aktuellen Plots als Bild fehlgeschlagen")));
    delete plotScreenshot;
}

void Deviation::on_button_SavePlotAsXY_clicked(){
    if ( (dst->saveXYFile(Enumerations::DeviationCurve, sst->prgset->defaultDir, this)) == Enumerations::OtherError)
        AuxiliaryMethods::errorMessage(this, new QString(tr("Speichern der aktuellen Abweichungskurve als X-Y-Datei fehlgeschlagen")));
}

void Deviation::on_button_Close_clicked(){
    mw->uncheck_buttonShowDeviation();
}


void Deviation::setSize(){

    RECT clientRect, windowRect;
    GetClientRect((HWND)(this->winId()), &clientRect);
    GetWindowRect((HWND)(this->winId()), &windowRect);

    int borderSize = ( ( (windowRect.right - windowRect.left) - (clientRect.right - clientRect.left) ) / 2);

    QDesktopWidget *desktop = new QDesktopWidget();
    int targetWidth = desktop->screenGeometry().width() - 2 * borderSize;    // maximum Width of mainWidget
    int targetHeight = desktop->screenGeometry().height() - ((windowRect.bottom - windowRect.top) - (clientRect.bottom - clientRect.top)) - 50; // maximum Height of mainWidget
    delete desktop;

    int targetWidthforCentering = targetWidth;
    int targetHeightforCentering = targetHeight;

    targetWidth = ( (targetWidth < sst->minSize_Deviation->x()) ? targetWidth : sst->minSize_Deviation->x() + (targetHeight < sst->minSize_Deviation->y() ? 30 : 0));
    targetHeight = ( (targetHeight < sst->minSize_Deviation->y()) ? targetHeight : sst->minSize_Deviation->y() + (targetWidth < sst->minSize_Deviation->x() ? 30 : 0));

    ui->centralWidget->resize(targetWidth,targetHeight);
    ui->mainWidget->resize(targetWidth - (targetHeight < sst->minSize_Deviation->y() ? 30 : 0), targetHeight - (targetWidth < sst->minSize_Deviation->x() ? 30 : 0));

    if (targetWidth < sst->minSize_Deviation->x()){
        ui->scrollBarH->setGeometry(0, ui->centralWidget->height() - 30, centralWidget()->width() - (targetHeight < sst->minSize_Deviation->y() ? 30 : 10), ui->scrollBarH->height());
        ui->scrollBarH->setMinimum(0);
        ui->scrollBarH->setMaximum(sst->minSize_Deviation->x() - ui->mainWidget->width());
        ui->scrollBarH->setSingleStep(10);
        ui->scrollBarH->setPageStep(10);
    }
    else{
        ui->scrollBarH->setDisabled(true);
        ui->scrollBarH->setVisible(false);
    }

    if (targetHeight < sst->minSize_Deviation->y()){
        ui->scrollBarV->setGeometry(ui->centralWidget->width() - 30, 0, ui->scrollBarV->width(), ui->centralWidget->height() - (targetWidth < sst->minSize_Deviation->x() ? 30 : 10));
        ui->scrollBarV->setMinimum(0);
        ui->scrollBarV->setMaximum(sst->minSize_Deviation->y() - ui->mainWidget->height());
        ui->scrollBarV->setSingleStep(10);
        ui->scrollBarV->setPageStep(10);
    }
    else{
        ui->scrollBarV->setDisabled(true);
        ui->scrollBarV->setVisible(false);
    }

    resize(targetWidth, targetHeight);
    move((targetWidthforCentering/2 - this->width()/2), ( (targetHeightforCentering)/2 - this->height()/2));
    setFixedSize(targetWidth, targetHeight);
    setWindowFlags( (windowFlags() | Qt::CustomizeWindowHint) & ~Qt::WindowMaximizeButtonHint);

}

void Deviation::showEvent(QShowEvent *event){
    if (!(event->spontaneous()))
        updatePlot();
    event->accept();
}

void Deviation::keyPressEvent(QKeyEvent *event){
    event->accept();
}

void Deviation::closeEvent(QCloseEvent *event){
    if (canClose)
        event->accept();
    else{
        event->ignore();
        ui->button_Close->click();
    }
}

